#include <limits>
#include <algorithm>

#include "Evaluation.hpp"

Score Evaluation::material(const Board& board) {
    Score result = 0;

    for (auto square: board.squares) {
        auto piece = square.second;
        if (piece.color == board.turn_color)
            result += Evaluation::piece(piece);
        else
            result -= Evaluation::piece(piece);
    }

    return result;
}

Score Evaluation::material_sum(const Board& board) {
    Score result = 0;

    for (auto square: board.squares) {
        auto piece = square.second;
        result += Evaluation::piece(piece);
    }

    return result;
}

UNumber coord_to_PTST(const Coord& coord) {
    return (8 - coord.rank) * 8 + (coord.file - 1);
}

Score square_cost[64] = {
    0,  1,  2,  3,  3,  2,  1,  0,
    1,  2,  4,  5,  5,  4,  2,  1,
    2,  4,  6,  7,  7,  6,  4,  2,
    3,  5,  7,  8,  8,  7,  5,  3,
    3,  5,  7,  8,  8,  7,  5,  3,
    2,  4,  6,  7,  7,  6,  4,  2,
    1,  2,  4,  5,  5,  4,  2,  1,
    0,  1,  2,  3,  3,  2,  1,  0
};

Score Evaluation::positional(const Board& board) {
    Score result = 0;

    for (auto square: board.squares) {
        auto coord = square.first;
        auto piece = square.second;
        if (piece.color == board.turn_color)
            result += Evaluation::PTST(piece, coord);
        else
            result -= Evaluation::PTST(piece, coord);
    }

    return result;
}

Score Evaluation::total(const Board& board) {
    return material(board) + positional(board);
}

Score white_pawn_PTST[64] = {
    0,  0,  0,  0,  0,  0,  0,  0,
    70, 70, 70, 70, 70, 70, 70, 70,
    50, 50, 50, 50, 50, 50, 50, 50,
    30, 30, 30, 30, 30, 30, 30, 30,
    0,  0,  10, 50, 50, 10, 0,  0,
    -5, -5, 20, 30, 30, 20, -5, -5,
    0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0
};

Score black_pawn_PTST[64] = {
    0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,
    -5, -5, 20, 30, 30, 20, -5, -5,
    0,  0,  10, 50, 50, 10, 0,  0,
    30, 30, 30, 30, 30, 30, 30, 30,
    50, 50, 50, 50, 50, 50, 50, 50,
    70, 70, 70, 70, 70, 70, 70, 70,
    0,  0,  0,  0,  0,  0,  0,  0
};

Score Evaluation::PTST(const Piece& piece, const Coord& coord) {
    if (piece.type == PAWN) {
        if (piece.color == WHITE)
            return white_pawn_PTST[coord_to_PTST(coord)];
        else
            return black_pawn_PTST[coord_to_PTST(coord)];
    }

    return 0;
}

// Based on http://home.comcast.net/~danheisman/Articles/evaluation_of_material_imbalance.htm
Score Evaluation::piece(const Piece& piece) {
    Score result =
        (piece.type == PAWN)?   1000:
        (piece.type == KNIGHT)? 3250:
        (piece.type == BISHOP)? 3250:
        (piece.type == ROOK)?   5000:
        (piece.type == QUEEN)?  9750:
        0; // for king

    return result;
}

const UNumber max_mate_halfmoves = 10000;

Score Evaluation::mate() {
    return std::numeric_limits<SNumber>::max();
}

bool Evaluation::is_mate(const Score& score) {
    if (score > 0)
        return mate() - score <= max_mate_halfmoves;
    else
        return (mate() + score >= 0) && (mate() + score <= 10000);
}

UNumber Evaluation::get_mate_half_moves(const Score& score) {
    if (score > 0)
        return mate() - score;
    else
        return -(mate() + score);
}
