#include "RestoreData.hpp"

Action Action::move(const Coord& from, const Coord& to) {
    Action result(ActionType::__MOVE);

    result.from = from;
    result.to = to;

    return result;
}

Action Action::set(const Piece& captured, const Coord& coords) {
    Action result(ActionType::__SET);

    result.captured = captured;
    result.capture_coord = coords;

    return result;
}

Action Action::remove(const Coord& coords) {
    Action result(ActionType::__REMOVE);

    result.capture_coord = coords;

    return result;
}

void Action::make(Board& board) {
    if (type == ActionType::__MOVE) {
        auto piece = board.get_piece_at(from);
        board.move_piece(from, to);
    } else if (type == ActionType::__SET) {
        board.add_piece(captured, capture_coord);
    } else if (type == ActionType::__REMOVE)  {
        board.remove_piece(capture_coord);
    } else throw 1;
}

RestoreData::RestoreData(const EnpassantPosibility& ep, const RookingPosibility& rp)
    :ep(ep), rp(rp)
{

}

void RestoreData::add_action(const Action& a) {
    (*this).restore_actions.push_back(a);
}

void RestoreData::restore(Board& board) {
    board.turn_color = board.turn_color.inverse();
    board.enpassant_posibility = ep;
    board.rooking_posibility = rp;
    board.last_moved = last_moved;

    for (auto action: restore_actions) {
        action.make(board);
    }
}

